@@ -17,15 +17,19 @@ ENUM_DW_TAG = dict(
     DW_TAG_entry_point                 = 0x03,
     DW_TAG_enumeration_type            = 0x04,
     DW_TAG_formal_parameter            = 0x05,
+    DW_TAG_global_subroutine           = 0x06, #DWARF1
+    DW_TAG_global_variable             = 0x07, #DWARF1
     DW_TAG_imported_declaration        = 0x08,
     DW_TAG_label                       = 0x0a,
     DW_TAG_lexical_block               = 0x0b,
+    DW_TAG_local_variable              = 0x0c, #DWARF1
     DW_TAG_member                      = 0x0d,
     DW_TAG_pointer_type                = 0x0f,
     DW_TAG_reference_type              = 0x10,
     DW_TAG_compile_unit                = 0x11,
     DW_TAG_string_type                 = 0x12,
     DW_TAG_structure_type              = 0x13,
+    DW_TAG_subroutine                  = 0x14, #DWARF1
     DW_TAG_subroutine_type             = 0x15,
     DW_TAG_typedef                     = 0x16,
     DW_TAG_union_type                  = 0x17,
@@ -111,6 +115,10 @@ ENUM_DW_AT = dict(
     DW_AT_sibling                   = 0x01,
     DW_AT_location                  = 0x02,
     DW_AT_name                      = 0x03,
+    DW_AT_fund_type                 = 0x05, #DWARF1
+    DW_AT_mod_fund_type             = 0x06, #DWARF1
+    DW_AT_user_def_type             = 0x07, #DWARF1
+    DW_AT_mod_u_d_type              = 0x08, #DWARF1
     DW_AT_ordering                  = 0x09,
     DW_AT_subscr_data               = 0x0a,
     DW_AT_byte_size                 = 0x0b,
@@ -136,7 +144,9 @@ ENUM_DW_AT = dict(
     DW_AT_is_optional               = 0x21,
     DW_AT_lower_bound               = 0x22,
     DW_AT_producer                  = 0x25,
+    DW_AT_protected                 = 0x26, #DWARF1
     DW_AT_prototyped                = 0x27,
+    DW_AT_public                    = 0x28, #DWARF1
     DW_AT_return_addr               = 0x2a,
     DW_AT_start_scope               = 0x2c,
     DW_AT_bit_stride                = 0x2e,
@@ -304,6 +314,7 @@ ENUM_DW_AT = dict(
 ENUM_DW_FORM = dict(
     DW_FORM_null                = 0x00,
     DW_FORM_addr                = 0x01,
+    DW_FORM_ref                 = 0x02, #DWARF1
     DW_FORM_block2              = 0x03,
     DW_FORM_block4              = 0x04,
     DW_FORM_data2               = 0x05,
